@@ -1,29 +1,32 @@
 import fnmatch
 import logging
+from typing import Any, Dict
 
 from django.contrib.auth import get_user_model
 from django.contrib.auth.models import Group
 from django.core.exceptions import ObjectDoesNotExist
 
+from glom import glom
 from mozilla_django_oidc.auth import (
     OIDCAuthenticationBackend as _OIDCAuthenticationBackend,
 )
 
-from .mixins import SoloConfigMixin
+from .mixins import GetAttributeMixin, SoloConfigMixin
+from .utils import obfuscate_claims
 
 logger = logging.getLogger(__name__)
 
 
-class OIDCAuthenticationBackend(SoloConfigMixin, _OIDCAuthenticationBackend):
+class OIDCAuthenticationBackend(
+    GetAttributeMixin, SoloConfigMixin, _OIDCAuthenticationBackend
+):
     """
     Modifies the default OIDCAuthenticationBackend to use a configurable claim
     as unique identifier (default `sub`).
     """
 
-    def __getattribute__(self, attr):
-        if attr.startswith("OIDC"):
-            return self.get_settings(attr, None)
-        return super().__getattribute__(attr)
+    config_identifier_field = "username_claim"
+    sensitive_claim_names = []
 
     def __init__(self, *args, **kwargs):
         self.UserModel = get_user_model()
@@ -36,25 +39,46 @@ class OIDCAuthenticationBackend(SoloConfigMixin, _OIDCAuthenticationBackend):
         # to avoid a large number of `OpenIDConnectConfig.get_solo` calls when
         # `OIDCAuthenticationBackend.__init__` is called for permission checks
 
+    def retrieve_identifier_claim(self, claims: dict) -> str:
+        # NOTE: this does not support the extraction of claims that contain dots "." in
+        # their name (e.g. {"foo.bar": "baz"})
+        identifier_claim_name = getattr(self.config, self.config_identifier_field)
+        unique_id = glom(claims, identifier_claim_name, default="")
+        return unique_id
+
+    def get_sensitive_claims_names(self) -> list:
+        """
+        Defines the claims that should be obfuscated before logging claims.
+        Nested claims can be specified by using a dotted path (e.g. "foo.bar.baz")
+
+        NOTE: this does not support claim names that have dots in them, so the following
+        claim cannot be marked as a sensitive claim
+
+            {
+                "foo.bar": "baz"
+            }
+        """
+        identifier_claim_name = getattr(self.config, self.config_identifier_field)
+        return [identifier_claim_name] + self.sensitive_claim_names
+
     def authenticate(self, *args, **kwargs):
         if not self.config.enabled:
             return None
 
         return super().authenticate(*args, **kwargs)
 
-    def get_user_instance_values(self, claims):
+    def get_user_instance_values(self, claims) -> Dict[str, Any]:
         """
         Map the names and values of the claims to the fields of the User model
         """
         return {
-            model_field: claims.get(claims_field, "")
+            model_field: glom(claims, claims_field, default="")
             for model_field, claims_field in self.config.claim_mapping.items()
         }
 
     def create_user(self, claims):
         """Return object for a newly created user account."""
-        username_claim = self.config.username_claim
-        unique_id = claims.get(username_claim)
+        unique_id = self.retrieve_identifier_claim(claims)
 
         logger.debug("Creating OIDC user: %s", unique_id)
 
@@ -67,8 +91,7 @@ class OIDCAuthenticationBackend(SoloConfigMixin, _OIDCAuthenticationBackend):
 
     def filter_users_by_claims(self, claims):
         """Return all users matching the specified subject."""
-        username_claim = self.config.username_claim
-        unique_id = claims.get(username_claim)
+        unique_id = self.retrieve_identifier_claim(claims)
 
         if not unique_id:
             return self.UserModel.objects.none()
@@ -76,18 +99,18 @@ class OIDCAuthenticationBackend(SoloConfigMixin, _OIDCAuthenticationBackend):
             **{f"{self.UserModel.USERNAME_FIELD}__iexact": unique_id}
         )
 
-    def verify_claims(self, claims):
+    def verify_claims(self, claims) -> bool:
         """Verify the provided claims to decide if authentication should be allowed."""
-        scopes = self.get_settings("OIDC_RP_SCOPES", "openid email")
+        claims_to_obfuscate = self.get_sensitive_claims_names()
+        obfuscated_claims = obfuscate_claims(claims, claims_to_obfuscate)
 
-        logger.debug("OIDC claims received: %s", claims)
+        logger.debug("OIDC claims received: %s", obfuscated_claims)
 
-        username_claim = self.config.username_claim
-
-        if username_claim not in claims:
+        identifier_claim_name = getattr(self.config, self.config_identifier_field)
+        if not glom(claims, identifier_claim_name, default=""):
             logger.error(
                 "%s not in OIDC claims, cannot proceed with authentication",
-                username_claim,
+                identifier_claim_name,
             )
             return False
         return True
@@ -121,9 +144,8 @@ class OIDCAuthenticationBackend(SoloConfigMixin, _OIDCAuthenticationBackend):
         if groups_claim:
             # Update the user's group memberships
             django_groups = [group.name for group in user.groups.all()]
-
-            if groups_claim in claims:
-                claim_groups = claims[groups_claim]
+            claim_groups = glom(claims, groups_claim, default=[])
+            if claim_groups:
                 if not isinstance(claim_groups, list):
                     claim_groups = [
                         claim_groups,
