@@ -1,11 +1,10 @@
-from typing import Dict
+from typing import Dict, List
 
 from django.conf import settings
 from django.contrib.auth import get_user_model
-from django.contrib.postgres.fields import JSONField
 from django.core.exceptions import ValidationError
 from django.db import models
-from django.utils.encoding import force_text
+from django.utils.encoding import force_str
 from django.utils.translation import gettext_lazy as _
 
 from django_better_admin_arrayfield.models.fields import ArrayField
@@ -13,8 +12,10 @@ from solo.models import SingletonModel, get_cache
 
 import mozilla_django_oidc_db.settings as oidc_settings
 
+from .compat import classproperty
 
-def get_default_scopes():
+
+def get_default_scopes() -> List[str]:
     """
     Returns the default scopes to request for OpenID Connect logins
     """
@@ -31,9 +32,65 @@ def get_claim_mapping() -> Dict[str, str]:
     }
 
 
-class OpenIDConnectConfig(SingletonModel):
+class CachingMixin:
+    @classmethod
+    def clear_cache(cls):
+        cache_name = getattr(
+            settings, "OIDC_CACHE", oidc_settings.MOZILLA_DJANGO_OIDC_DB_CACHE
+        )
+        if cache_name:
+            cache = get_cache(cache_name)
+            cache_key = cls.get_cache_key()
+            cache.delete(cache_key)
+
+    def set_to_cache(self):
+        cache_name = getattr(
+            settings,
+            "MOZILLA_DJANGO_OIDC_DB_CACHE",
+            oidc_settings.MOZILLA_DJANGO_OIDC_DB_CACHE,
+        )
+        if not cache_name:
+            return None
+        cache = get_cache(cache_name)
+        cache_key = self.get_cache_key()
+        timeout = getattr(
+            settings,
+            "MOZILLA_DJANGO_OIDC_DB_CACHE_TIMEOUT",
+            oidc_settings.MOZILLA_DJANGO_OIDC_DB_CACHE_TIMEOUT,
+        )
+        cache.set(cache_key, self, timeout)
+
+    @classmethod
+    def get_cache_key(cls) -> str:
+        prefix = cls.custom_oidc_db_prefix or getattr(
+            settings,
+            "MOZILLA_DJANGO_OIDC_DB_PREFIX",
+            oidc_settings.MOZILLA_DJANGO_OIDC_DB_PREFIX,
+        )
+        return "%s:%s" % (prefix, cls.__name__.lower())
+
+    @classmethod
+    def get_solo(cls) -> SingletonModel:
+        cache_name = getattr(
+            settings,
+            "MOZILLA_DJANGO_OIDC_DB_CACHE",
+            oidc_settings.MOZILLA_DJANGO_OIDC_DB_CACHE,
+        )
+        if not cache_name:
+            obj, created = cls.objects.get_or_create(pk=cls.singleton_instance_id)
+            return obj
+        cache = get_cache(cache_name)
+        cache_key = cls.get_cache_key()
+        obj = cache.get(cache_key)
+        if not obj:
+            obj, created = cls.objects.get_or_create(pk=cls.singleton_instance_id)
+            obj.set_to_cache()
+        return obj
+
+
+class OpenIDConnectConfigBase(SingletonModel):
     """
-    Configuration for authentication/authorization via OpenID connect
+    Defines the required fields for a config to establish an OIDC connection
     """
 
     enabled = models.BooleanField(
@@ -83,7 +140,8 @@ class OpenIDConnectConfig(SingletonModel):
         _("JSON Web Key Set endpoint"),
         max_length=1000,
         help_text=_(
-            "URL of your OpenID Connect provider JSON Web Key Set endpoint. Required if `RS256` is used as signing algorithm"
+            "URL of your OpenID Connect provider JSON Web Key Set endpoint. "
+            "Required if `RS256` is used as signing algorithm."
         ),
         blank=True,
     )
@@ -106,18 +164,72 @@ class OpenIDConnectConfig(SingletonModel):
         _("Sign key"),
         max_length=1000,
         help_text=_(
-            "Key the Identity Provider uses to sign ID tokens in the case of an RSA sign algorithm. Should be the signing key in PEM or DER format"
+            "Key the Identity Provider uses to sign ID tokens in the case of an RSA sign algorithm. "
+            "Should be the signing key in PEM or DER format."
         ),
         blank=True,
     )
 
+    # Advanced settings
+    oidc_use_nonce = models.BooleanField(
+        _("Use nonce"),
+        help_text=_(
+            "Controls whether the OpenID Connect client uses nonce verification"
+        ),
+        default=True,
+    )
+    oidc_nonce_size = models.PositiveIntegerField(
+        _("Nonce size"),
+        help_text=_(
+            "Sets the length of the random string used for OpenID Connect nonce verification"
+        ),
+        default=32,
+    )
+    oidc_state_size = models.PositiveIntegerField(
+        _("State size"),
+        help_text=_(
+            "Sets the length of the random string used for OpenID Connect state verification"
+        ),
+        default=32,
+    )
+    oidc_exempt_urls = ArrayField(
+        verbose_name=_("URLs exempt from session renewal"),
+        base_field=models.CharField(_("Exempt URL"), max_length=1000),
+        default=list,
+        blank=True,
+        help_text=_(
+            "This is a list of absolute url paths, regular expressions for url paths, "
+            "or Django view names. This plus the mozilla-django-oidc urls are exempted "
+            "from the session renewal by the SessionRefresh middleware."
+        ),
+    )
+
+    @property
+    def oidc_rp_scopes(self) -> str:
+        """
+        Scopes should be formatted as a string with spaces
+        """
+        return " ".join(self.oidc_rp_scopes_list)
+
+    class Meta:
+        abstract = True
+
+    def __str__(self) -> str:
+        return force_str(self._meta.verbose_name)
+
+
+class OpenIDConnectConfig(CachingMixin, OpenIDConnectConfigBase):
+    """
+    Configuration for authentication/authorization via OpenID connect
+    """
+
     username_claim = models.CharField(
         _("username claim"),
         max_length=50,
         default="sub",
         help_text=_("The name of the OIDC claim that is used as the username"),
     )
-    claim_mapping = JSONField(
+    claim_mapping = models.JSONField(
         _("claim mapping"),
         default=get_claim_mapping,
         help_text=("Mapping from user-model fields to OIDC claims"),
@@ -152,16 +264,14 @@ class OpenIDConnectConfig(SingletonModel):
         _("make users staff"),
         default=False,
         help_text=_(
-            "Users will be flagged as being a staff user automatically. This allows users to login to the admin interface. By default they have no permissions, even if they are staff."
+            "Users will be flagged as being a staff user automatically. This allows "
+            "users to login to the admin interface. By default they have no permissions, even if they are staff."
         ),
     )
 
     class Meta:
         verbose_name = _("OpenID Connect configuration")
 
-    def __str__(self):
-        return force_text(self._meta.verbose_name)
-
     def clean(self):
         super().clean()
 
@@ -188,63 +298,9 @@ class OpenIDConnectConfig(SingletonModel):
                 }
             )
 
-    @property
-    def oidc_rp_scopes(self):
+    @classproperty
+    def custom_oidc_db_prefix(cls) -> str:
         """
-        Scopes should be formatted as a string with spaces
+        Cache prefix that can be overridden
         """
-        return " ".join(self.oidc_rp_scopes_list)
-
-    @classmethod
-    def clear_cache(cls):
-        cache_name = getattr(
-            settings, "OIDC_CACHE", oidc_settings.MOZILLA_DJANGO_OIDC_DB_CACHE
-        )
-        if cache_name:
-            cache = get_cache(cache_name)
-            cache_key = cls.get_cache_key()
-            cache.delete(cache_key)
-
-    def set_to_cache(self):
-        cache_name = getattr(
-            settings,
-            "MOZILLA_DJANGO_OIDC_DB_CACHE",
-            oidc_settings.MOZILLA_DJANGO_OIDC_DB_CACHE,
-        )
-        if not cache_name:
-            return None
-        cache = get_cache(cache_name)
-        cache_key = self.get_cache_key()
-        timeout = getattr(
-            settings,
-            "MOZILLA_DJANGO_OIDC_DB_CACHE_TIMEOUT",
-            oidc_settings.MOZILLA_DJANGO_OIDC_DB_CACHE_TIMEOUT,
-        )
-        cache.set(cache_key, self, timeout)
-
-    @classmethod
-    def get_cache_key(cls):
-        prefix = getattr(
-            settings,
-            "MOZILLA_DJANGO_OIDC_DB_PREFIX",
-            oidc_settings.MOZILLA_DJANGO_OIDC_DB_PREFIX,
-        )
-        return "%s:%s" % (prefix, cls.__name__.lower())
-
-    @classmethod
-    def get_solo(cls):
-        cache_name = getattr(
-            settings,
-            "MOZILLA_DJANGO_OIDC_DB_CACHE",
-            oidc_settings.MOZILLA_DJANGO_OIDC_DB_CACHE,
-        )
-        if not cache_name:
-            obj, created = cls.objects.get_or_create(pk=cls.singleton_instance_id)
-            return obj
-        cache = get_cache(cache_name)
-        cache_key = cls.get_cache_key()
-        obj = cache.get(cache_key)
-        if not obj:
-            obj, created = cls.objects.get_or_create(pk=cls.singleton_instance_id)
-            obj.set_to_cache()
-        return obj
+        return oidc_settings.MOZILLA_DJANGO_OIDC_DB_PREFIX
