@@ -23,6 +23,8 @@ def test_derive_endpoints_success():
         "groups_claim": "roles",
         "sync_groups_glob_pattern": "*",
         "username_claim": "sub",
+        "oidc_nonce_size": 32,
+        "oidc_state_size": 32,
     }
     form = OpenIDConnectConfigForm(data=form_data)
 
@@ -69,6 +71,8 @@ def test_derive_endpoints_extra_field():
         "groups_claim": "roles",
         "sync_groups_glob_pattern": "*",
         "username_claim": "sub",
+        "oidc_nonce_size": 32,
+        "oidc_state_size": 32,
     }
 
     class ExtendedOpenIDConnectConfigForm(OpenIDConnectConfigForm):
@@ -114,6 +118,8 @@ def test_derive_endpoints_request_error(*m):
         "groups_claim": "roles",
         "sync_groups_glob_pattern": "*",
         "username_claim": "sub",
+        "oidc_nonce_size": 32,
+        "oidc_state_size": 32,
     }
     form = OpenIDConnectConfigForm(data=form_data)
 
@@ -137,6 +143,8 @@ def test_derive_endpoints_json_error(*m):
         "groups_claim": "roles",
         "sync_groups_glob_pattern": "*",
         "username_claim": "sub",
+        "oidc_nonce_size": 32,
+        "oidc_state_size": 32,
     }
     form = OpenIDConnectConfigForm(data=form_data)
 
@@ -158,6 +166,8 @@ def test_no_discovery_endpoint_other_fields_required():
         "groups_claim": "roles",
         "sync_groups_glob_pattern": "*",
         "username_claim": "sub",
+        "oidc_nonce_size": 32,
+        "oidc_state_size": 32,
     }
     form = OpenIDConnectConfigForm(data=form_data)
 
